@@ -26,17 +26,16 @@ from ufl.compound_expressions import cross_expr, determinant_expr, inverse_expr
 from ufl.core.multiindex import Index, indices
 from ufl.corealg.map_dag import map_expr_dag
 from ufl.corealg.multifunction import MultiFunction, memoized_handler
-# FacetJacobianInverse,
-# FacetOrientation, QuadratureWeight,
 from ufl.domain import extract_unique_domain
-from ufl.log import error
 from ufl.measure import custom_integral_types, point_integral_types
 from ufl.operators import conj, max_value, min_value, real, sqrt
 from ufl.tensors import as_tensor, as_vector
 
 
 class GeometryLoweringApplier(MultiFunction):
+    """Geometry lowering."""
     def __init__(self, preserve_types=()):
+        """Initialise."""
         MultiFunction.__init__(self)
         # Store preserve_types as boolean lookup table
         self._preserve_types = [False] * Expr._ufl_num_typecodes_
@@ -46,15 +45,17 @@ class GeometryLoweringApplier(MultiFunction):
     expr = MultiFunction.reuse_if_untouched
 
     def terminal(self, t):
+        """Apply to terminal."""
         return t
 
     @memoized_handler
     def jacobian(self, o):
+        """Apply to jacobian."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
         domain = extract_unique_domain(o)
         if domain.ufl_coordinate_element().mapping() != "identity":
-            error("Piola mapped coordinates are not implemented.")
+            raise ValueError("Piola mapped coordinates are not implemented.")
         # Note: No longer supporting domain.coordinates(), always
         # preserving SpatialCoordinate object.  However if Jacobians
         # are not preserved, using
@@ -64,6 +65,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def _future_jacobian(self, o):
+        """Apply to _future_jacobian."""
         # If we're not using Coefficient to represent the spatial
         # coordinate, we can just as well just return o here too
         # unless we add representation of basis functions and dofs to
@@ -72,6 +74,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def jacobian_inverse(self, o):
+        """Apply to jacobian_inverse."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -84,6 +87,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def jacobian_determinant(self, o):
+        """Apply to jacobian_determinant."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -102,6 +106,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def facet_jacobian(self, o):
+        """Apply to facet_jacobian."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -113,6 +118,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def facet_jacobian_inverse(self, o):
+        """Apply to facet_jacobian_inverse."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -124,6 +130,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def facet_jacobian_determinant(self, o):
+        """Apply to facet_jacobian_determinant."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -142,18 +149,24 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def spatial_coordinate(self, o):
-        "Fall through to coordinate field of domain if it exists."
+        """Apply to spatial_coordinate.
+
+        Fall through to coordinate field of domain if it exists.
+        """
         if self._preserve_types[o._ufl_typecode_]:
             return o
         if extract_unique_domain(o).ufl_coordinate_element().mapping() != "identity":
-            error("Piola mapped coordinates are not implemented.")
+            raise ValueError("Piola mapped coordinates are not implemented.")
         # No longer supporting domain.coordinates(), always preserving
         # SpatialCoordinate object.
         return o
 
     @memoized_handler
     def cell_coordinate(self, o):
-        "Compute from physical coordinates if they are known, using the appropriate mappings."
+        """Apply to cell_coordinate.
+
+        Compute from physical coordinates if they are known, using the appropriate mappings.
+        """
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -167,14 +180,16 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def facet_cell_coordinate(self, o):
+        """Apply to facet_cell_coordinate."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
-        error("Missing computation of facet reference coordinates "
-              "from physical coordinates via mappings.")
+        raise ValueError("Missing computation of facet reference coordinates "
+                         "from physical coordinates via mappings.")
 
     @memoized_handler
     def cell_volume(self, o):
+        """Apply to cell_volume."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -191,6 +206,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def facet_area(self, o):
+        """Apply to facet_area."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -212,13 +228,14 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def circumradius(self, o):
+        """Apply to circumradius."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
         domain = extract_unique_domain(o)
 
         if not domain.is_piecewise_linear_simplex_domain():
-            error("Circumradius only makes sense for affine simplex cells")
+            raise ValueError("Circumradius only makes sense for affine simplex cells")
 
         cellname = domain.ufl_cell().cellname()
         cellvolume = self.cell_volume(CellVolume(domain))
@@ -252,13 +269,16 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def max_cell_edge_length(self, o):
+        """Apply to max_cell_edge_length."""
         return self._reduce_cell_edge_length(o, max_value)
 
     @memoized_handler
     def min_cell_edge_length(self, o):
+        """Apply to min_cell_edge_length."""
         return self._reduce_cell_edge_length(o, min_value)
 
     def _reduce_cell_edge_length(self, o, reduction_op):
+        """Apply to _reduce_cell_edge_length."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -283,6 +303,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def cell_diameter(self, o):
+        """Apply to cell_diameter."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -307,20 +328,23 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def max_facet_edge_length(self, o):
+        """Apply to max_facet_edge_length."""
         return self._reduce_facet_edge_length(o, max_value)
 
     @memoized_handler
     def min_facet_edge_length(self, o):
+        """Apply to min_facet_edge_length."""
         return self._reduce_facet_edge_length(o, min_value)
 
     def _reduce_facet_edge_length(self, o, reduction_op):
+        """Apply to _reduce_facet_edge_length."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
         domain = extract_unique_domain(o)
 
         if domain.ufl_cell().topological_dimension() < 3:
-            error("Facet edge lengths only make sense for topological dimension >= 3.")
+            raise ValueError("Facet edge lengths only make sense for topological dimension >= 3.")
 
         elif not domain.ufl_coordinate_element().degree() == 1:
             # Don't lower bendy cells, instead leave it to form compiler
@@ -337,6 +361,7 @@ class GeometryLoweringApplier(MultiFunction):
 
     @memoized_handler
     def cell_normal(self, o):
+        """Apply to cell_normal."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -358,17 +383,18 @@ class GeometryLoweringApplier(MultiFunction):
                 # to the 'right')
                 cell_normal = as_vector((-J[1, 0], J[0, 0]))
             else:
-                error("Cell normal not implemented for tdim %d, gdim %d" % (tdim, gdim))
+                raise ValueError(f"Cell normal not implemented for tdim {tdim}, gdim {gdim}")
 
             # Return normalized vector, sign corrected by cell
             # orientation
             co = CellOrientation(domain)
             return co * cell_normal / sqrt(cell_normal[i] * cell_normal[i])
         else:
-            error("What do you want cell normal in gdim={0}, tdim={1} to be?".format(gdim, tdim))
+            raise ValueError(f"Cell normal undefined for tdim {tdim}, gdim {gdim}")
 
     @memoized_handler
     def facet_normal(self, o):
+        """Apply to facet_normal."""
         if self._preserve_types[o._ufl_typecode_]:
             return o
 
@@ -409,7 +435,7 @@ class GeometryLoweringApplier(MultiFunction):
             r = n
 
         if r.ufl_shape != o.ufl_shape:
-            error("Inconsistent dimensions (in=%d, out=%d)." % (o.ufl_shape[0], r.ufl_shape[0]))
+            raise ValueError(f"Inconsistent dimensions (in={o.ufl_shape[0]}, out={r.ufl_shape[0]}).")
         return r
 
 
@@ -418,8 +444,9 @@ def apply_geometry_lowering(form, preserve_types=()):
 
     Assumes the expression is preprocessed or at least that derivatives have been expanded.
 
-    @param form:
-        An Expr or Form.
+    Args:
+        form: An Expr or Form.
+        preserve_types: Preserved types
     """
     if isinstance(form, Form):
         newintegrals = [apply_geometry_lowering(integral, preserve_types)
@@ -444,4 +471,4 @@ def apply_geometry_lowering(form, preserve_types=()):
         return map_expr_dag(mf, expr)
 
     else:
-        error("Invalid type %s" % (form.__class__.__name__,))
+        raise ValueError(f"Invalid type {form.__class__.__name__}")
