@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """This module contains algorithms for traversing expression trees in different ways."""
 
 # Copyright (C) 2008-2016 Martin Sandve Alnæs and Anders Logg
@@ -9,7 +8,6 @@
 #
 # Modified by Anders Logg, 2008
 
-from ufl.log import error
 from ufl.core.expr import Expr
 from ufl.integral import Integral
 from ufl.action import Action
@@ -20,8 +18,8 @@ from ufl.form import Form, FormSum, BaseForm
 # --- Traversal utilities ---
 
 def iter_expressions(a):
-    """Utility function to handle Form, Integral and any Expr
-    the same way when inspecting expressions.
+    """Utility function to handle Form, Integral and any Expr the same way when inspecting expressions.
+
     Returns an iterable over Expr instances:
     - a is an Expr: (a,)
     - a is an Integral:  the integrand expression of a
@@ -38,4 +36,4 @@ def iter_expressions(a):
         return tuple(e for op in a.ufl_operands for e in iter_expressions(op))
     elif isinstance(a, (Expr, BaseForm)):
         return (a,)
-    error("Not an UFL type: %s" % str(type(a)))
+    raise ValueError(f"Not an UFL type: {type(a)}")
