@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-"This module defines the UFL finite element classes."
-
+"""This module defines the UFL finite element classes."""
 # Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of UFL (https://www.fenicsproject.org)
@@ -12,10 +10,8 @@
 # Modified by Anders Logg 2014
 # Modified by Massimiliano Leoni, 2016
 
-from ufl.log import error
 from ufl.permutation import compute_indices
 from ufl.utils.sequences import product, max_degree
-from ufl.utils.dicts import EmptyDict
 from ufl.utils.indexflattening import flatten_multiindex, unflatten_index, shape_to_strides
 from ufl.cell import as_cell
 
@@ -24,16 +20,14 @@ from ufl.finiteelement.finiteelement import FiniteElement
 
 
 class MixedElement(FiniteElementBase):
-    """A finite element composed of a nested hierarchy of mixed or simple
-    elements."""
+    """A finite element composed of a nested hierarchy of mixed or simple elements."""
     __slots__ = ("_sub_elements", "_cells")
 
     def __init__(self, *elements, **kwargs):
-        "Create mixed finite element from given list of elements"
-
+        """Create mixed finite element from given list of elements."""
         if type(self) is MixedElement:
             if kwargs:
-                error("Not expecting keyword arguments to MixedElement constructor.")
+                raise ValueError("Not expecting keyword arguments to MixedElement constructor.")
 
         # Un-nest arguments if we get a single argument with a list of elements
         if len(elements) == 1 and isinstance(elements[0], (tuple, list)):
@@ -50,7 +44,7 @@ class MixedElement(FiniteElementBase):
             cell = cells[0]
             # Require that all elements are defined on the same cell
             if not all(c == cell for c in cells[1:]):
-                error("Sub elements must live on the same cell.")
+                raise ValueError("Sub elements must live on the same cell.")
         else:
             cell = None
 
@@ -61,7 +55,7 @@ class MixedElement(FiniteElementBase):
         else:
             quad_scheme = elements[0].quadrature_scheme()
             if not all(e.quadrature_scheme() == quad_scheme for e in elements):
-                error("Quadrature scheme mismatch for sub elements of mixed element.")
+                raise ValueError("Quadrature scheme mismatch for sub elements of mixed element.")
 
         # Compute value sizes in global and reference configurations
         value_size_sum = sum(product(s.value_shape()) for s in self._sub_elements)
@@ -81,8 +75,8 @@ class MixedElement(FiniteElementBase):
             # This is not valid for tensor elements with symmetries,
             # assume subclasses deal with their own validation
             if product(value_shape) != value_size_sum:
-                error("Provided value_shape doesn't match the "
-                      "total value size of all subelements.")
+                raise ValueError("Provided value_shape doesn't match the "
+                                 "total value size of all subelements.")
 
         # Initialize element data
         degrees = {e.degree() for e in self._sub_elements} - {None}
@@ -91,19 +85,26 @@ class MixedElement(FiniteElementBase):
                                    value_shape, reference_value_shape)
 
     def __repr__(self):
+        """Doc."""
         return "MixedElement(" + ", ".join(repr(e) for e in self._sub_elements) + ")"
 
+    def _is_linear(self):
+        """Doc."""
+        return all(i._is_linear() for i in self._sub_elements)
+
     def reconstruct_from_elements(self, *elements):
-        "Reconstruct a mixed element from new subelements."
+        """Reconstruct a mixed element from new subelements."""
         if all(a == b for (a, b) in zip(elements, self._sub_elements)):
             return self
         return MixedElement(*elements)
 
     def symmetry(self):
-        """Return the symmetry dict, which is a mapping :math:`c_0 \\to c_1`
+        r"""Return the symmetry dict, which is a mapping :math:`c_0 \\to c_1`.
+
         meaning that component :math:`c_0` is represented by component
         :math:`c_1`.
-        A component is a tuple of one or more ints."""
+        A component is a tuple of one or more ints.
+        """
         # Build symmetry map from symmetries of subelements
         sm = {}
         # Base index of the current subelement into mixed value
@@ -120,29 +121,33 @@ class MixedElement(FiniteElementBase):
             # Update base index for next element
             j += product(sh)
         if j != product(self.value_shape()):
-            error("Size mismatch in symmetry algorithm.")
-        return sm or EmptyDict
+            raise ValueError("Size mismatch in symmetry algorithm.")
+        return sm or {}
 
     def sobolev_space(self):
+        """Doc."""
         return max(e.sobolev_space() for e in self._sub_elements)
 
     def mapping(self):
+        """Doc."""
         if all(e.mapping() == "identity" for e in self._sub_elements):
             return "identity"
         else:
             return "undefined"
 
     def num_sub_elements(self):
-        "Return number of sub elements."
+        """Return number of sub elements."""
         return len(self._sub_elements)
 
     def sub_elements(self):
-        "Return list of sub elements."
+        """Return list of sub elements."""
         return self._sub_elements
 
     def extract_subelement_component(self, i):
-        """Extract direct subelement index and subelement relative
-        component index for a given component index."""
+        """Extract direct subelement index and subelement relative.
+
+        component index for a given component index.
+        """
         if isinstance(i, int):
             i = (i,)
         self._check_component(i)
@@ -161,7 +166,7 @@ class MixedElement(FiniteElementBase):
                     break
                 j -= si
             if j < 0:
-                error("Moved past last value component!")
+                raise ValueError("Moved past last value component!")
 
             # Convert index into a shape tuple
             st = shape_to_strides(sh)
@@ -171,19 +176,23 @@ class MixedElement(FiniteElementBase):
             # index is first axis
             sub_element_index = i[0]
             if sub_element_index >= len(self._sub_elements):
-                error("Illegal component index (dimension %d)." % sub_element_index)
+                raise ValueError(f"Illegal component index (dimension {sub_element_index}).")
             component = i[1:]
         return (sub_element_index, component)
 
     def extract_component(self, i):
-        """Recursively extract component index relative to a (simple) element
-        and that element for given value component index."""
+        """Recursively extract component index relative to a (simple) element.
+
+        and that element for given value component index.
+        """
         sub_element_index, component = self.extract_subelement_component(i)
         return self._sub_elements[sub_element_index].extract_component(component)
 
     def extract_subelement_reference_component(self, i):
-        """Extract direct subelement index and subelement relative
-        reference_component index for a given reference_component index."""
+        """Extract direct subelement index and subelement relative.
+
+        reference_component index for a given reference_component index.
+        """
         if isinstance(i, int):
             i = (i,)
         self._check_reference_component(i)
@@ -201,7 +210,7 @@ class MixedElement(FiniteElementBase):
                 break
             j -= si
         if j < 0:
-            error("Moved past last value reference_component!")
+            raise ValueError("Moved past last value reference_component!")
 
         # Convert index into a shape tuple
         st = shape_to_strides(sh)
@@ -209,14 +218,15 @@ class MixedElement(FiniteElementBase):
         return (sub_element_index, reference_component)
 
     def extract_reference_component(self, i):
-        """Recursively extract reference_component index relative to a (simple) element
-        and that element for given value reference_component index."""
+        """Recursively extract reference_component index relative to a (simple) element.
+
+        and that element for given value reference_component index.
+        """
         sub_element_index, reference_component = self.extract_subelement_reference_component(i)
         return self._sub_elements[sub_element_index].extract_reference_component(reference_component)
 
     def is_cellwise_constant(self, component=None):
-        """Return whether the basis functions of this
-        element is spatially constant over each cell."""
+        """Return whether the basis functions of this element is spatially constant over each cell."""
         if component is None:
             return all(e.is_cellwise_constant() for e in self.sub_elements())
         else:
@@ -224,7 +234,7 @@ class MixedElement(FiniteElementBase):
             return e.is_cellwise_constant()
 
     def degree(self, component=None):
-        "Return polynomial degree of finite element."
+        """Return polynomial degree of finite element."""
         if component is None:
             return self._degree  # from FiniteElementBase, computed as max of subelements in __init__
         else:
@@ -232,9 +242,11 @@ class MixedElement(FiniteElementBase):
             return e.degree()
 
     def reconstruct(self, **kwargs):
+        """Doc."""
         return MixedElement(*[e.reconstruct(**kwargs) for e in self.sub_elements()])
 
     def variant(self):
+        """Doc."""
         try:
             variant, = {e.variant() for e in self.sub_elements()}
             return variant
@@ -242,44 +254,24 @@ class MixedElement(FiniteElementBase):
             return None
 
     def __str__(self):
-        "Format as string for pretty printing."
+        """Format as string for pretty printing."""
         tmp = ", ".join(str(element) for element in self._sub_elements)
         return "<Mixed element: (" + tmp + ")>"
 
     def shortstr(self):
-        "Format as string for pretty printing."
+        """Format as string for pretty printing."""
         tmp = ", ".join(element.shortstr() for element in self._sub_elements)
         return "Mixed<" + tmp + ">"
 
 
 class VectorElement(MixedElement):
-    "A special case of a mixed finite element where all elements are equal."
+    """A special case of a mixed finite element where all elements are equal."""
 
     __slots__ = ("_repr", "_mapping", "_sub_element")
 
     def __init__(self, family, cell=None, degree=None, dim=None,
                  form_degree=None, quad_scheme=None, variant=None):
-        """
-        Create vector element (repeated mixed element)
-
-        *Arguments*
-            family (string)
-               The finite element family (or an existing FiniteElement)
-            cell
-               The geometric cell, ignored if family is a FiniteElement
-            degree (int)
-               The polynomial degree, ignored if family is a FiniteElement
-            dim (int)
-               The value dimension of the element (optional)
-            form_degree (int)
-               The form degree (FEEC notation, used when field is
-               viewed as k-form), ignored if family is a FiniteElement
-            quad_scheme
-               The quadrature scheme (optional), ignored if family is a FiniteElement
-            variant
-               Hint for the local basis function variant (optional)
-        """
-
+        """Create vector element (repeated mixed element)."""
         if isinstance(family, FiniteElementBase):
             sub_element = family
             cell = sub_element.cell()
@@ -296,7 +288,7 @@ class VectorElement(MixedElement):
         # Set default size if not specified
         if dim is None:
             if cell is None:
-                error("Cannot infer vector dimension without a cell.")
+                raise ValueError("Cannot infer vector dimension without a cell.")
             dim = cell.geometric_dimension()
 
         self._mapping = sub_element.mapping()
@@ -310,8 +302,10 @@ class VectorElement(MixedElement):
         # Initialize element data
         MixedElement.__init__(self, sub_elements, value_shape=value_shape,
                               reference_value_shape=reference_value_shape)
-        FiniteElementBase.__init__(self, sub_element.family(), cell, sub_element.degree(), quad_scheme,
-                                   value_shape, reference_value_shape)
+
+        FiniteElementBase.__init__(self, sub_element.family(), sub_element.cell(), sub_element.degree(),
+                                   sub_element.quadrature_scheme(), value_shape, reference_value_shape)
+
         self._sub_element = sub_element
 
         if variant is None:
@@ -323,9 +317,11 @@ class VectorElement(MixedElement):
         self._repr = f"VectorElement({repr(sub_element)}, dim={dim}{var_str})"
 
     def __repr__(self):
+        """Doc."""
         return self._repr
 
     def reconstruct(self, **kwargs):
+        """Doc."""
         sub_element = self._sub_element.reconstruct(**kwargs)
         return VectorElement(sub_element, dim=len(self.sub_elements()))
 
@@ -334,24 +330,22 @@ class VectorElement(MixedElement):
         return self._sub_element.variant()
 
     def mapping(self):
+        """Doc."""
         return self._mapping
 
     def __str__(self):
-        "Format as string for pretty printing."
+        """Format as string for pretty printing."""
         return ("<vector element with %d components of %s>" %
                 (len(self._sub_elements), self._sub_element))
 
     def shortstr(self):
-        "Format as string for pretty printing."
+        """Format as string for pretty printing."""
         return "Vector<%d x %s>" % (len(self._sub_elements),
                                     self._sub_element.shortstr())
 
 
 class TensorElement(MixedElement):
-    """A special case of a mixed finite element where all elements are
-    equal.
-
-    """
+    """A special case of a mixed finite element where all elements are equal."""
     __slots__ = ("_sub_element", "_shape", "_symmetry",
                  "_sub_element_mapping",
                  "_flattened_sub_element_mapping",
@@ -359,18 +353,7 @@ class TensorElement(MixedElement):
 
     def __init__(self, family, cell=None, degree=None, shape=None,
                  symmetry=None, quad_scheme=None, variant=None):
-        """Create tensor element (repeated mixed element with optional symmetries).
-
-        :arg family: The family string, or an existing FiniteElement.
-        :arg cell: The geometric cell (ignored if family is a FiniteElement).
-        :arg degree: The polynomial degree (ignored if family is a FiniteElement).
-        :arg shape: The shape of the element (defaults to a square
-             tensor given by the geometric dimension of the cell).
-        :arg symmetry: Optional symmetries.
-        :arg quad_scheme: Optional quadrature scheme (ignored if
-             family is a FiniteElement).
-        :arg variant:  Hint for the local basis function variant (optional)"""
-
+        """Create tensor element (repeated mixed element with optional symmetries)."""
         if isinstance(family, FiniteElementBase):
             sub_element = family
             cell = sub_element.cell()
@@ -385,29 +368,29 @@ class TensorElement(MixedElement):
         # Set default shape if not specified
         if shape is None:
             if cell is None:
-                error("Cannot infer tensor shape without a cell.")
+                raise ValueError("Cannot infer tensor shape without a cell.")
             dim = cell.geometric_dimension()
             shape = (dim, dim)
 
         if symmetry is None:
-            symmetry = EmptyDict
+            symmetry = {}
         elif symmetry is True:
             # Construct default symmetry dict for matrix elements
             if not (len(shape) == 2 and shape[0] == shape[1]):
-                error("Cannot set automatic symmetry for non-square tensor.")
+                raise ValueError("Cannot set automatic symmetry for non-square tensor.")
             symmetry = dict(((i, j), (j, i)) for i in range(shape[0])
                             for j in range(shape[1]) if i > j)
         else:
             if not isinstance(symmetry, dict):
-                error("Expecting symmetry to be None (unset), True, or dict.")
+                raise ValueError("Expecting symmetry to be None (unset), True, or dict.")
 
         # Validate indices in symmetry dict
         for i, j in symmetry.items():
             if len(i) != len(j):
-                error("Non-matching length of symmetry index tuples.")
+                raise ValueError("Non-matching length of symmetry index tuples.")
             for k in range(len(i)):
                 if not (i[k] >= 0 and j[k] >= 0 and i[k] < shape[k] and j[k] < shape[k]):
-                    error("Symmetry dimensions out of bounds.")
+                    raise ValueError("Symmetry dimensions out of bounds.")
 
         # Compute all index combinations for given shape
         indices = compute_indices(shape)
@@ -463,6 +446,7 @@ class TensorElement(MixedElement):
                       f"symmetry={symmetry}{var_str})")
 
     def __repr__(self):
+        """Doc."""
         return self._repr
 
     def variant(self):
@@ -470,14 +454,18 @@ class TensorElement(MixedElement):
         return self._sub_element.variant()
 
     def mapping(self):
+        """Doc."""
         return self._mapping
 
     def flattened_sub_element_mapping(self):
+        """Doc."""
         return self._flattened_sub_element_mapping
 
     def extract_subelement_component(self, i):
-        """Extract direct subelement index and subelement relative
-        component index for a given component index."""
+        """Extract direct subelement index and subelement relative.
+
+        component index for a given component index.
+        """
         if isinstance(i, int):
             i = (i,)
         self._check_component(i)
@@ -487,23 +475,26 @@ class TensorElement(MixedElement):
         ii = i[:l]
         jj = i[l:]
         if ii not in self._sub_element_mapping:
-            error("Illegal component index %s." % (i,))
+            raise ValueError(f"Illegal component index {i}.")
         k = self._sub_element_mapping[ii]
         return (k, jj)
 
     def symmetry(self):
-        """Return the symmetry dict, which is a mapping :math:`c_0 \\to c_1`
+        r"""Return the symmetry dict, which is a mapping :math:`c_0 \\to c_1`.
+
         meaning that component :math:`c_0` is represented by component
         :math:`c_1`.
-        A component is a tuple of one or more ints."""
+        A component is a tuple of one or more ints.
+        """
         return self._symmetry
 
     def reconstruct(self, **kwargs):
+        """Doc."""
         sub_element = self._sub_element.reconstruct(**kwargs)
         return TensorElement(sub_element, shape=self._shape, symmetry=self._symmetry)
 
     def __str__(self):
-        "Format as string for pretty printing."
+        """Format as string for pretty printing."""
         if self._symmetry:
             tmp = ", ".join("%s -> %s" % (a, b) for (a, b) in self._symmetry.items())
             sym = " with symmetries (%s)" % tmp
@@ -513,7 +504,7 @@ class TensorElement(MixedElement):
                 (self.value_shape(), self._sub_element, sym))
 
     def shortstr(self):
-        "Format as string for pretty printing."
+        """Format as string for pretty printing."""
         if self._symmetry:
             tmp = ", ".join("%s -> %s" % (a, b) for (a, b) in self._symmetry.items())
             sym = " with symmetries (%s)" % tmp
