@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """The Measure class."""
 
 # Copyright (C) 2008-2016 Martin Sandve Alnæs
@@ -14,13 +13,11 @@ import numbers
 
 from itertools import chain
 
-from ufl.log import error
 from ufl.core.expr import Expr
 from ufl.checks import is_true_ufl_scalar
 from ufl.constantvalue import as_ufl
-from ufl.utils.dicts import EmptyDict
 from ufl.domain import as_domain, AbstractDomain, extract_domains
-from ufl.protocols import id_or_none, metadata_equal, metadata_hashdata
+from ufl.protocols import id_or_none
 
 
 # Export list for ufl.classes
@@ -33,36 +30,31 @@ __all_classes__ = ["Measure", "MeasureSum", "MeasureProduct"]
 # Enumeration of valid domain types
 _integral_types = [
     # === Integration over full topological dimension:
-    ("cell", "dx"),                     # Over cells of a mesh
-    # ("macro_cell", "dE"),              # Over a group of adjacent cells (TODO: Arbitrary cell group? Not currently used.)
+    ("cell", "dx"),  # Over cells of a mesh
 
     # === Integration over topological dimension - 1:
-    ("exterior_facet", "ds"),           # Over one-sided exterior facets of a mesh
-    ("interior_facet", "dS"),           # Over two-sided facets between pairs of adjacent cells of a mesh
+    ("exterior_facet", "ds"),  # Over one-sided exterior facets of a mesh
+    ("interior_facet", "dS"),  # Over two-sided facets between pairs of adjacent cells of a mesh
 
     # === Integration over topological dimension 0
-    ("vertex", "dP"),                    # Over vertices of a mesh
-    # ("vertex", "dV"),                  # TODO: Use this over vertices?
-    # ("point", "dP"),                   # TODO: Use this over arbitrary points inside cells?
+    ("vertex", "dP"),  # Over vertices of a mesh
 
     # === Integration over custom domains
-    ("custom", "dc"),                 # Over custom user-defined domains (run-time quadrature points)
-    ("cutcell", "dC"),                # Over a cell with some part cut away (run-time quadrature points)
-    ("interface", "dI"),              # Over a facet fragment overlapping with two or more cells (run-time quadrature points)
-    ("overlap", "dO"),                # Over a cell fragment overlapping with two or more cells (run-time quadrature points)
+    ("custom", "dc"),  # Over custom user-defined domains (run-time quadrature points)
+    ("cutcell", "dC"),  # Over a cell with some part cut away (run-time quadrature points)
+    ("interface", "dI"),  # Over a facet fragment overlapping with two or more cells (run-time quadrature points)
+    ("overlap", "dO"),  # Over a cell fragment overlapping with two or more cells (run-time quadrature points)
 
-    # === Firedrake specific hacks on the way out:
-    # TODO: Remove these, firedrake can use metadata instead
-    # and create the measure objects in firedrake:
+    # === Firedrake specifics:
     ("exterior_facet_bottom", "ds_b"),  # Over bottom facets on extruded mesh
-    ("exterior_facet_top", "ds_t"),     # Over top facets on extruded mesh
-    ("exterior_facet_vert", "ds_v"),    # Over side facets of an extruded mesh
-    ("interior_facet_horiz", "dS_h"),   # Over horizontal facets of an extruded mesh
+    ("exterior_facet_top", "ds_t"),  # Over top facets on extruded mesh
+    ("exterior_facet_vert", "ds_v"),  # Over side facets of an extruded mesh
+    ("interior_facet_horiz", "dS_h"),  # Over horizontal facets of an extruded mesh
     ("interior_facet_vert", "dS_v"),  # Over vertical facets of an extruded mesh
 ]
 
-integral_type_to_measure_name = dict((l, s) for l, s in _integral_types)
-measure_name_to_integral_type = dict((s, l) for l, s in _integral_types)
+integral_type_to_measure_name = {i: s for i, s in _integral_types}
+measure_name_to_integral_type = {s: i for i, s in _integral_types}
 
 custom_integral_types = ("custom", "cutcell", "interface", "overlap")
 point_integral_types = ("vertex",)  # "point")
@@ -70,75 +62,62 @@ facet_integral_types = ("exterior_facet", "interior_facet")
 
 
 def register_integral_type(integral_type, measure_name):
+    """Register an integral type."""
     global integral_type_to_measure_name, measure_name_to_integral_type
     if measure_name != integral_type_to_measure_name.get(integral_type, measure_name):
-        error("Integral type already added with different measure name!")
+        raise ValueError("Integral type already added with different measure name!")
     if integral_type != measure_name_to_integral_type.get(measure_name, integral_type):
-        error("Measure name already used for another domain type!")
+        raise ValueError("Measure name already used for another domain type!")
     integral_type_to_measure_name[integral_type] = measure_name
     measure_name_to_integral_type[measure_name] = integral_type
 
 
 def as_integral_type(integral_type):
-    "Map short name to long name and require a valid one."
+    """Map short name to long name and require a valid one."""
     integral_type = integral_type.replace(" ", "_")
     integral_type = measure_name_to_integral_type.get(integral_type,
                                                       integral_type)
     if integral_type not in integral_type_to_measure_name:
-        error("Invalid integral_type.")
+        raise ValueError("Invalid integral_type.")
     return integral_type
 
 
 def integral_types():
-    "Return a tuple of all domain type strings."
+    """Return a tuple of all domain type strings."""
     return tuple(sorted(integral_type_to_measure_name.keys()))
 
 
 def measure_names():
-    "Return a tuple of all measure name strings."
+    """Return a tuple of all measure name strings."""
     return tuple(sorted(measure_name_to_integral_type.keys()))
 
 
 class Measure(object):
-    __slots__ = ("_integral_type",
-                 "_domain",
-                 "_subdomain_id",
-                 "_metadata",
-                 "_subdomain_data")
     """Representation of an integration measure.
 
     The Measure object holds information about integration properties
     to be transferred to a Form on multiplication with a scalar
     expression.
-
     """
 
+    __slots__ = ("_integral_type", "_domain", "_subdomain_id", "_metadata", "_subdomain_data")
+
     def __init__(self,
                  integral_type,  # "dx" etc
                  domain=None,
                  subdomain_id="everywhere",
                  metadata=None,
                  subdomain_data=None):
-        """
-        integral_type:
-            str, one of "cell", etc.,
-            or short form "dx", etc.
+        """Initialise.
 
-        domain:
-            an AbstractDomain object (most often a Mesh)
-
-        subdomain_id:
-            either string "everywhere",
-            a single subdomain id int,
-            or tuple of ints
-
-        metadata:
-            dict, with additional compiler-specific parameters
-            affecting how code is generated, including parameters
-            for optimization or debugging of generated code.
-
-        subdomain_data:
-            object representing data to interpret subdomain_id with.
+        Args:
+            integral_type: one of "cell", etc, or short form "dx", etc
+            domain: an AbstractDomain object (most often a Mesh)
+            subdomain_id: either string "everywhere", a single subdomain id int, or tuple of ints
+            metadata: dict, with additional compiler-specific parameters
+                affecting how code is generated, including parameters
+                for optimization or debugging of generated code
+            subdomain_data: object representing data to interpret subdomain_id with
         """
         # Map short name to long name and require a valid one
         self._integral_type = as_integral_type(integral_type)
@@ -146,30 +125,30 @@ class Measure(object):
         # Check that we either have a proper AbstractDomain or none
         self._domain = None if domain is None else as_domain(domain)
         if not (self._domain is None or isinstance(self._domain, AbstractDomain)):
-            error("Invalid domain.")
+            raise ValueError("Invalid domain.")
 
         # Store subdomain data
         self._subdomain_data = subdomain_data
         # FIXME: Cannot require this (yet) because we currently have
         # no way to implement ufl_id for dolfin SubDomain
         # if not (self._subdomain_data is None or hasattr(self._subdomain_data, "ufl_id")):
-        #     error("Invalid domain data, missing ufl_id() implementation.")
+        #     raise ValueError("Invalid domain data, missing ufl_id() implementation.")
 
         # Accept "everywhere", single subdomain, or multiple
         # subdomains
         if isinstance(subdomain_id, tuple):
             for did in subdomain_id:
                 if not isinstance(did, numbers.Integral):
-                    error("Invalid subdomain_id %s." % (did,))
+                    raise ValueError(f"Invalid subdomain_id {did}.")
         else:
             if not (subdomain_id in ("everywhere",) or isinstance(subdomain_id, numbers.Integral)):
-                error("Invalid subdomain_id %s." % (subdomain_id,))
+                raise ValueError(f"Invalid subdomain_id {subdomain_id}.")
         self._subdomain_id = subdomain_id
 
         # Validate compiler options are None or dict
         if metadata is not None and not isinstance(metadata, dict):
-            error("Invalid metadata.")
-        self._metadata = metadata or EmptyDict
+            raise ValueError("Invalid metadata.")
+        self._metadata = metadata or {}
 
     def integral_type(self):
         """Return the domain type.
@@ -186,14 +165,15 @@ class Measure(object):
         return self._domain
 
     def subdomain_id(self):
-        "Return the domain id of this measure (integer)."
+        """Return the domain id of this measure (integer)."""
         return self._subdomain_id
 
     def metadata(self):
-        """Return the integral metadata. This data is not interpreted by UFL.
+        """Return the integral metadata.
+
+        This data is not interpreted by UFL.
         It is passed to the form compiler which can ignore it or use
         it to compile each integral of a form in a different way.
-
         """
         return self._metadata
 
@@ -203,11 +183,9 @@ class Measure(object):
                     domain=None,
                     metadata=None,
                     subdomain_data=None):
-        """Construct a new Measure object with some properties replaced with
-        new values.
+        """Construct a new Measure object with some properties replaced with new values.
 
         Example:
-        -------
             <dm = Measure instance>
             b = dm.reconstruct(subdomain_id=2)
             c = dm.reconstruct(metadata={ "quadrature_degree": 3 })
@@ -215,7 +193,6 @@ class Measure(object):
         Used by the call operator, so this is equivalent:
             b = dm(2)
             c = dm(0, { "quadrature_degree": 3 })
-
         """
         if subdomain_id is None:
             subdomain_id = self.subdomain_id()
@@ -230,10 +207,11 @@ class Measure(object):
                        metadata=metadata, subdomain_data=subdomain_data)
 
     def subdomain_data(self):
-        """Return the integral subdomain_data. This data is not interpreted by
+        """Return the integral subdomain_data.
+
+        This data is not interpreted by
         UFL.  Its intension is to give a context in which the domain
         id is interpreted.
-
         """
         return self._subdomain_data
 
@@ -243,7 +221,6 @@ class Measure(object):
     def __call__(self, subdomain_id=None, metadata=None, domain=None,
                  subdomain_data=None, degree=None, scheme=None):
         """Reconfigure measure with new domain specification or metadata."""
-
         # Let syntax dx() mean integral over everywhere
         all_args = (subdomain_id, metadata, domain, subdomain_data,
                     degree, scheme)
@@ -253,11 +230,11 @@ class Measure(object):
         # Let syntax dx(domain) or dx(domain, metadata) mean integral
         # over entire domain.  To do this we need to hijack the first
         # argument:
-        if subdomain_id is not None and (isinstance(subdomain_id,
-                                                    AbstractDomain) or
-                                         hasattr(subdomain_id, 'ufl_domain')):
+        if subdomain_id is not None and (
+            isinstance(subdomain_id, AbstractDomain) or hasattr(subdomain_id, 'ufl_domain')
+        ):
             if domain is not None:
-                error("Ambiguous: setting domain both as keyword argument and first argument.")
+                raise ValueError("Ambiguous: setting domain both as keyword argument and first argument.")
             subdomain_id, domain = "everywhere", as_domain(subdomain_id)
 
         # If degree or scheme is set, inject into metadata. This is a
@@ -278,7 +255,7 @@ class Measure(object):
                                 subdomain_data=subdomain_data)
 
     def __str__(self):
-        global integral_type_to_measure_name
+        """Format as a string."""
         name = integral_type_to_measure_name[self._integral_type]
         args = []
 
@@ -286,7 +263,7 @@ class Measure(object):
             args.append("subdomain_id=%s" % (self._subdomain_id,))
         if self._domain is not None:
             args.append("domain=%s" % (self._domain,))
-        if self._metadata:  # Stored as EmptyDict if None
+        if self._metadata:  # Stored as {} if None
             args.append("metadata=%s" % (self._metadata,))
         if self._subdomain_data is not None:
             args.append("subdomain_data=%s" % (self._subdomain_data,))
@@ -294,9 +271,7 @@ class Measure(object):
         return "%s(%s)" % (name, ', '.join(args))
 
     def __repr__(self):
-        "Return a repr string for this Measure."
-        global integral_type_to_measure_name
-
+        """Return a repr string for this Measure."""
         args = []
         args.append(repr(self._integral_type))
 
@@ -304,7 +279,7 @@ class Measure(object):
             args.append("subdomain_id=%s" % repr(self._subdomain_id))
         if self._domain is not None:
             args.append("domain=%s" % repr(self._domain))
-        if self._metadata:  # Stored as EmptyDict if None
+        if self._metadata:  # Stored as {} if None
             args.append("metadata=%s" % repr(self._metadata))
         if self._subdomain_data is not None:
             args.append("subdomain_data=%s" % repr(self._subdomain_data))
@@ -313,28 +288,29 @@ class Measure(object):
         return r
 
     def __hash__(self):
-        "Return a hash value for this Measure."
+        """Return a hash value for this Measure."""
+        metadata_hashdata = tuple(sorted((k, id(v)) for k, v in list(self._metadata.items())))
         hashdata = (self._integral_type,
                     self._subdomain_id,
                     hash(self._domain),
-                    metadata_hashdata(self._metadata),
+                    metadata_hashdata,
                     id_or_none(self._subdomain_data))
         return hash(hashdata)
 
     def __eq__(self, other):
-        "Checks if two Measures are equal."
-        return (isinstance(other, Measure) and
-                self._integral_type == other._integral_type and
-                self._subdomain_id == other._subdomain_id and
-                self._domain == other._domain and
-                id_or_none(self._subdomain_data) == id_or_none(other._subdomain_data) and
-                metadata_equal(self._metadata, other._metadata))
+        """Checks if two Measures are equal."""
+        sorted_metadata = sorted((k, id(v)) for k, v in list(self._metadata.items()))
+        sorted_other_metadata = sorted((k, id(v)) for k, v in list(other._metadata.items()))
+
+        return (isinstance(other, Measure) and self._integral_type == other._integral_type and  # noqa: W504
+                self._subdomain_id == other._subdomain_id and self._domain == other._domain and  # noqa: W504
+                id_or_none(self._subdomain_data) == id_or_none(other._subdomain_data) and  # noqa: W504
+                sorted_metadata == sorted_other_metadata)
 
     def __add__(self, other):
         """Add two measures (self+other).
 
         Creates an intermediate object used for the notation
-
           expr * (dx(1) + dx(2)) := expr * dx(1) + expr * dx(2)
         """
         if isinstance(other, Measure):
@@ -348,7 +324,6 @@ class Measure(object):
         """Multiply two measures (self*other).
 
         Creates an intermediate object used for the notation
-
           expr * (dm1 * dm2) := expr * dm1 * dm2
 
         This is work in progress and not functional.
@@ -361,15 +336,12 @@ class Measure(object):
             return NotImplemented
 
     def __rmul__(self, integrand):
-        """Multiply a scalar expression with measure to construct a form with
-        a single integral.
+        """Multiply a scalar expression with measure to construct a form with a single integral.
 
         This is to implement the notation
-
             form = integrand * self
 
         Integration properties are taken from this Measure object.
-
         """
         # Avoid circular imports
         from ufl.integral import Integral
@@ -386,9 +358,10 @@ class Measure(object):
 
         # Allow only scalar integrands
         if not is_true_ufl_scalar(integrand):
-            error("Can only integrate scalar expressions. The integrand is a "
-                  "tensor expression with value shape %s and free indices with labels %s." %
-                  (integrand.ufl_shape, integrand.ufl_free_indices))
+            raise ValueError(
+                "Can only integrate scalar expressions. The integrand is a "
+                f"tensor expression with value shape {integrand.ufl_shape} and "
+                f"free indices with labels {integrand.ufl_free_indices}.")
 
         # If we have a tuple of domain ids build the integrals one by
         # one and construct as a Form in one go.
@@ -400,7 +373,7 @@ class Measure(object):
         # Check that we have an integer subdomain or a string
         # ("everywhere" or "otherwise", any more?)
         if not isinstance(subdomain_id, (str, numbers.Integral,)):
-            error("Expecting integer or string domain id.")
+            raise ValueError("Expecting integer or string domain id.")
 
         # If we don't have an integration domain, try to find one in
         # integrand
@@ -410,9 +383,9 @@ class Measure(object):
             if len(domains) == 1:
                 domain, = domains
             elif len(domains) == 0:
-                error("This integral is missing an integration domain.")
+                raise ValueError("This integral is missing an integration domain.")
             else:
-                error("Multiple domains found, making the choice of integration domain ambiguous.")
+                raise ValueError("Multiple domains found, making the choice of integration domain ambiguous.")
 
         # Otherwise create and return a one-integral form
         integral = Integral(integrand=integrand,
@@ -428,23 +401,24 @@ class MeasureSum(object):
     """Represents a sum of measures.
 
     This is a notational intermediate object to translate the notation
-
         f*(ds(1)+ds(3))
-
     into
-
         f*ds(1) + f*ds(3)
     """
+
     __slots__ = ("_measures",)
 
     def __init__(self, *measures):
+        """Initialise."""
         self._measures = measures
 
     def __rmul__(self, other):
+        """Multiply."""
         integrals = [other * m for m in self._measures]
         return sum(integrals)
 
     def __add__(self, other):
+        """Add."""
         if isinstance(other, Measure):
             return MeasureSum(*(self._measures + (other,)))
         elif isinstance(other, MeasureSum):
@@ -452,6 +426,7 @@ class MeasureSum(object):
         return NotImplemented
 
     def __str__(self):
+        """Format as a string."""
         return "{\n    " + "\n  + ".join(map(str, self._measures)) + "\n}"
 
 
@@ -464,22 +439,21 @@ class MeasureProduct(object):
 
     This is work in progress and not functional. It needs support
     in other parts of ufl and the rest of the code generation chain.
-
     """
+
     __slots__ = ("_measures",)
 
     def __init__(self, *measures):
-        "Create MeasureProduct from given list of measures."
+        """Create MeasureProduct from given list of measures."""
         self._measures = measures
         if len(self._measures) < 2:
-            error("Expecting at least two measures.")
+            raise ValueError("Expecting at least two measures.")
 
     def __mul__(self, other):
         """Flatten multiplication of product measures.
 
         This is to ensure that (dm1*dm2)*dm3 is stored as a simple
         list (dm1,dm2,dm3) in a single MeasureProduct.
-
         """
         if isinstance(other, Measure):
             measures = self.sub_measures() + [other]
@@ -488,8 +462,10 @@ class MeasureProduct(object):
             return NotImplemented
 
     def __rmul__(self, integrand):
-        error("TODO: Implement MeasureProduct.__rmul__ to construct integral and form somehow.")
+        """Multiply."""
+        # TODO: Implement MeasureProduct.__rmul__ to construct integral and form somehow.
+        raise NotImplementedError()
 
     def sub_measures(self):
-        "Return submeasures."
+        """Return submeasures."""
         return self._measures
